@@ -3,15 +3,18 @@ This script populates tox.ini automatically using release data from PyPI.
 """
 
 import functools
+import hashlib
 import os
 import sys
 import time
 from bisect import bisect_left
 from collections import defaultdict
+from datetime import datetime, timedelta, timezone  # noqa: F401
 from importlib.metadata import metadata
 from packaging.specifiers import SpecifierSet
 from packaging.version import Version
 from pathlib import Path
+from textwrap import dedent
 from typing import Optional, Union
 
 # Adding the scripts directory to PATH. This is necessary in order to be able
@@ -26,6 +29,10 @@ from config import TEST_SUITE_CONFIG
 from split_tox_gh_actions.split_tox_gh_actions import GROUPS
 
 
+# Set CUTOFF this to a datetime to ignore packages older than CUTOFF
+CUTOFF = None
+# CUTOFF = datetime.now(tz=timezone.utc) - timedelta(days=365 * 5)
+
 TOX_FILE = Path(__file__).resolve().parent.parent.parent / "tox.ini"
 ENV = Environment(
     loader=FileSystemLoader(Path(__file__).resolve().parent),
@@ -33,6 +40,8 @@ ENV = Environment(
     lstrip_blocks=True,
 )
 
+PYPI_COOLDOWN = 0.15  # seconds to wait between requests to PyPI
+
 PYPI_PROJECT_URL = "https://pypi.python.org/pypi/{project}/json"
 PYPI_VERSION_URL = "https://pypi.python.org/pypi/{project}/{version}/json"
 CLASSIFIER_PREFIX = "Programming Language :: Python :: "
@@ -46,23 +55,26 @@ IGNORE = {
     # suites over to this script. Some entries will probably stay forever
     # as they don't fit the mold (e.g. common, asgi, which don't have a 3rd party
     # pypi package to install in different versions).
+    #
+    # Test suites that will have to remain hardcoded since they don't fit the
+    # toxgen usecase
+    "asgi",
+    "aws_lambda",
+    "cloud_resource_context",
     "common",
     "gevent",
     "opentelemetry",
     "potel",
+    # Integrations that can be migrated -- we should eventually remove all
+    # of these from the IGNORE list
     "aiohttp",
     "anthropic",
     "arq",
-    "asgi",
     "asyncpg",
-    "aws_lambda",
     "beam",
     "boto3",
     "chalice",
     "cohere",
-    "cloud_resource_context",
-    "cohere",
-    "django",
     "fastapi",
     "gcp",
     "httpx",
@@ -70,7 +82,6 @@ IGNORE = {
     "huggingface_hub",
     "langchain",
     "langchain_notiktoken",
-    "litestar",
     "openai",
     "openai_notiktoken",
     "pure_eval",
@@ -83,30 +94,39 @@ IGNORE = {
 }
 
 
+def fetch_url(url: str) -> Optional[dict]:
+    for attempt in range(3):
+        pypi_data = requests.get(url)
+
+        if pypi_data.status_code == 200:
+            return pypi_data.json()
+
+        backoff = PYPI_COOLDOWN * 2**attempt
+        print(
+            f"{url} returned an error: {pypi_data.status_code}. Attempt {attempt + 1}/3. Waiting {backoff}s"
+        )
+        time.sleep(backoff)
+
+    return None
+
+
 @functools.cache
-def fetch_package(package: str) -> dict:
+def fetch_package(package: str) -> Optional[dict]:
     """Fetch package metadata from PyPI."""
     url = PYPI_PROJECT_URL.format(project=package)
-    pypi_data = requests.get(url)
-
-    if pypi_data.status_code != 200:
-        print(f"{package} not found")
-
-    return pypi_data.json()
+    return fetch_url(url)
 
 
 @functools.cache
-def fetch_release(package: str, version: Version) -> dict:
+def fetch_release(package: str, version: Version) -> Optional[dict]:
+    """Fetch release metadata from PyPI."""
     url = PYPI_VERSION_URL.format(project=package, version=version)
-    pypi_data = requests.get(url)
-
-    if pypi_data.status_code != 200:
-        print(f"{package} not found")
-
-    return pypi_data.json()
+    return fetch_url(url)
 
 
-def _prefilter_releases(integration: str, releases: dict[str, dict]) -> list[Version]:
+def _prefilter_releases(
+    integration: str, releases: dict[str, dict], older_than: Optional[datetime] = None
+) -> tuple[list[Version], Optional[Version]]:
     """
     Filter `releases`, removing releases that are for sure unsupported.
 
@@ -115,6 +135,10 @@ def _prefilter_releases(integration: str, releases: dict[str, dict]) -> list[Ver
     they require additional API calls to be made. The purpose of this function is
     to slim down the list so that we don't have to make more API calls than
     necessary for releases that are for sure not supported.
+
+    The function returns a tuple with:
+    - the list of prefiltered releases
+    - an optional prerelease if there is one that should be tested
     """
     min_supported = _MIN_VERSIONS.get(integration)
     if min_supported is not None:
@@ -124,7 +148,14 @@ def _prefilter_releases(integration: str, releases: dict[str, dict]) -> list[Ver
             f"  {integration} doesn't have a minimum version defined in sentry_sdk/integrations/__init__.py. Consider defining one"
         )
 
+    include_versions = None
+    if TEST_SUITE_CONFIG[integration].get("include") is not None:
+        include_versions = SpecifierSet(
+            TEST_SUITE_CONFIG[integration]["include"], prereleases=True
+        )
+
     filtered_releases = []
+    last_prerelease = None
 
     for release, data in releases.items():
         if not data:
@@ -135,14 +166,28 @@ def _prefilter_releases(integration: str, releases: dict[str, dict]) -> list[Ver
         if meta["yanked"]:
             continue
 
+        uploaded = datetime.fromisoformat(meta["upload_time_iso_8601"])
+
+        if older_than is not None and uploaded > older_than:
+            continue
+
+        if CUTOFF is not None and uploaded < CUTOFF:
+            continue
+
         version = Version(release)
 
         if min_supported and version < min_supported:
             continue
 
-        if version.is_prerelease or version.is_postrelease:
-            # TODO: consider the newest prerelease unless obsolete
-            # https://github.com/getsentry/sentry-python/issues/4030
+        if version.is_postrelease or version.is_devrelease:
+            continue
+
+        if include_versions is not None and version not in include_versions:
+            continue
+
+        if version.is_prerelease:
+            if last_prerelease is None or version > last_prerelease:
+                last_prerelease = version
             continue
 
         for i, saved_version in enumerate(filtered_releases):
@@ -157,22 +202,41 @@ def _prefilter_releases(integration: str, releases: dict[str, dict]) -> list[Ver
         else:
             filtered_releases.append(version)
 
-    return sorted(filtered_releases)
+    filtered_releases.sort()
+
+    # Check if the latest prerelease is relevant (i.e., it's for a version higher
+    # than the last released version); if not, don't consider it
+    if last_prerelease is not None:
+        if not filtered_releases or last_prerelease > filtered_releases[-1]:
+            return filtered_releases, last_prerelease
+
+    return filtered_releases, None
 
 
-def get_supported_releases(integration: str, pypi_data: dict) -> list[Version]:
+def get_supported_releases(
+    integration: str, pypi_data: dict, older_than: Optional[datetime] = None
+) -> tuple[list[Version], Optional[Version]]:
     """
     Get a list of releases that are currently supported by the SDK.
 
     This takes into account a handful of parameters (Python support, the lowest
     version we've defined for the framework, the date of the release).
+
+    We return the list of supported releases and optionally also the newest
+    prerelease, if it should be tested (meaning it's for a version higher than
+    the current stable version).
+
+    If an `older_than` timestamp is provided, no release newer than that will be
+    considered.
     """
     package = pypi_data["info"]["name"]
 
     # Get a consolidated list without taking into account Python support yet
     # (because that might require an additional API call for some
     # of the releases)
-    releases = _prefilter_releases(integration, pypi_data["releases"])
+    releases, latest_prerelease = _prefilter_releases(
+        integration, pypi_data["releases"], older_than
+    )
 
     # Determine Python support
     expected_python_versions = TEST_SUITE_CONFIG[integration].get("python")
@@ -182,8 +246,14 @@ def get_supported_releases(integration: str, pypi_data: dict) -> list[Version]:
         expected_python_versions = SpecifierSet(f">={MIN_PYTHON_VERSION}")
 
     def _supports_lowest(release: Version) -> bool:
-        time.sleep(0.1)  # don't DoS PYPI
-        py_versions = determine_python_versions(fetch_release(package, release))
+        time.sleep(PYPI_COOLDOWN)  # don't DoS PYPI
+
+        pypi_data = fetch_release(package, release)
+        if pypi_data is None:
+            print("Failed to fetch necessary data from PyPI. Aborting.")
+            sys.exit(1)
+
+        py_versions = determine_python_versions(pypi_data)
         target_python_versions = TEST_SUITE_CONFIG[integration].get("python")
         if target_python_versions:
             target_python_versions = SpecifierSet(target_python_versions)
@@ -196,14 +266,18 @@ def get_supported_releases(integration: str, pypi_data: dict) -> list[Version]:
             # version(s) that we do, cut off the rest
             releases = releases[i:]
 
-    return releases
+    return releases, latest_prerelease
 
 
-def pick_releases_to_test(releases: list[Version]) -> list[Version]:
+def pick_releases_to_test(
+    releases: list[Version], last_prerelease: Optional[Version]
+) -> list[Version]:
     """Pick a handful of releases to test from a sorted list of supported releases."""
     # If the package has majors (or major-like releases, even if they don't do
     # semver), we want to make sure we're testing them all. If not, we just pick
     # the oldest, the newest, and a couple in between.
+    #
+    # If there is a relevant prerelease, also test that in addition to the above.
     has_majors = len(set([v.major for v in releases])) > 1
     filtered_releases = set()
 
@@ -238,7 +312,11 @@ def pick_releases_to_test(releases: list[Version]) -> list[Version]:
             releases[-1],  # latest
         }
 
-    return sorted(filtered_releases)
+    filtered_releases = sorted(filtered_releases)
+    if last_prerelease is not None:
+        filtered_releases.append(last_prerelease)
+
+    return filtered_releases
 
 
 def supported_python_versions(
@@ -381,7 +459,9 @@ def _render_dependencies(integration: str, releases: list[Version]) -> list[str]
     return rendered
 
 
-def write_tox_file(packages: dict) -> None:
+def write_tox_file(
+    packages: dict, update_timestamp: bool, last_updated: datetime
+) -> None:
     template = ENV.get_template("tox.jinja")
 
     context = {"groups": {}}
@@ -400,6 +480,11 @@ def write_tox_file(packages: dict) -> None:
                 }
             )
 
+    if update_timestamp:
+        context["updated"] = datetime.now(tz=timezone.utc).isoformat()
+    else:
+        context["updated"] = last_updated.isoformat()
+
     rendered = template.render(context)
 
     with open(TOX_FILE, "w") as file:
@@ -437,7 +522,11 @@ def _add_python_versions_to_release(
     integration: str, package: str, release: Version
 ) -> None:
     release_pypi_data = fetch_release(package, release)
-    time.sleep(0.1)  # give PYPI some breathing room
+    if release_pypi_data is None:
+        print("Failed to fetch necessary data from PyPI. Aborting.")
+        sys.exit(1)
+
+    time.sleep(PYPI_COOLDOWN)  # give PYPI some breathing room
 
     target_python_versions = TEST_SUITE_CONFIG[integration].get("python")
     if target_python_versions:
@@ -453,7 +542,59 @@ def _add_python_versions_to_release(
     release.rendered_python_versions = _render_python_versions(release.python_versions)
 
 
-def main() -> None:
+def get_file_hash() -> str:
+    """Calculate a hash of the tox.ini file."""
+    hasher = hashlib.md5()
+
+    with open(TOX_FILE, "rb") as f:
+        buf = f.read()
+        hasher.update(buf)
+
+    return hasher.hexdigest()
+
+
+def get_last_updated() -> Optional[datetime]:
+    timestamp = None
+
+    with open(TOX_FILE, "r") as f:
+        for line in f:
+            if line.startswith("# Last generated:"):
+                timestamp = datetime.fromisoformat(line.strip().split()[-1])
+                break
+
+    if timestamp is None:
+        print(
+            "Failed to find out when tox.ini was last generated; the timestamp seems to be missing from the file."
+        )
+
+    return timestamp
+
+
+def main(fail_on_changes: bool = False) -> None:
+    """
+    Generate tox.ini from the tox.jinja template.
+
+    The script has two modes of operation:
+    - fail on changes mode (if `fail_on_changes` is True)
+    - normal mode (if `fail_on_changes` is False)
+
+    Fail on changes mode is run on every PR to make sure that `tox.ini`,
+    `tox.jinja` and this script don't go out of sync because of manual changes
+    in one place but not the other.
+
+    Normal mode is meant to be run as a cron job, regenerating tox.ini and
+    proposing the changes via a PR.
+    """
+    print(f"Running in {'fail_on_changes' if fail_on_changes else 'normal'} mode.")
+    last_updated = get_last_updated()
+    if fail_on_changes:
+        # We need to make the script ignore any new releases after the `last_updated`
+        # timestamp so that we don't fail CI on a PR just because a new package
+        # version was released, leading to unrelated changes in tox.ini.
+        print(
+            f"Since we're in fail_on_changes mode, we're only considering releases before the last tox.ini update at {last_updated.isoformat()}."
+        )
+
     global MIN_PYTHON_VERSION, MAX_PYTHON_VERSION
     sdk_python_versions = _parse_python_versions_from_classifiers(
         metadata("sentry-sdk").get_all("Classifier")
@@ -478,9 +619,19 @@ def main() -> None:
 
             # Fetch data for the main package
             pypi_data = fetch_package(package)
+            if pypi_data is None:
+                print("Failed to fetch necessary data from PyPI. Aborting.")
+                sys.exit(1)
 
             # Get the list of all supported releases
-            releases = get_supported_releases(integration, pypi_data)
+
+            # If in fail-on-changes mode, ignore releases newer than `last_updated`
+            older_than = last_updated if fail_on_changes else None
+
+            releases, latest_prerelease = get_supported_releases(
+                integration, pypi_data, older_than
+            )
+
             if not releases:
                 print("  Found no supported releases.")
                 continue
@@ -488,9 +639,9 @@ def main() -> None:
             _compare_min_version_with_defined(integration, releases)
 
             # Pick a handful of the supported releases to actually test against
-            # and fetch the PYPI data for each to determine which Python versions
+            # and fetch the PyPI data for each to determine which Python versions
             # to test it on
-            test_releases = pick_releases_to_test(releases)
+            test_releases = pick_releases_to_test(releases, latest_prerelease)
 
             for release in test_releases:
                 _add_python_versions_to_release(integration, package, release)
@@ -510,8 +661,44 @@ def main() -> None:
                     }
                 )
 
-    write_tox_file(packages)
+    if fail_on_changes:
+        old_file_hash = get_file_hash()
+
+    write_tox_file(
+        packages, update_timestamp=not fail_on_changes, last_updated=last_updated
+    )
+
+    if fail_on_changes:
+        new_file_hash = get_file_hash()
+        if old_file_hash != new_file_hash:
+            raise RuntimeError(
+                dedent(
+                    """
+                Detected that `tox.ini` is out of sync with
+                `scripts/populate_tox/tox.jinja` and/or
+                `scripts/populate_tox/populate_tox.py`. This might either mean
+                that `tox.ini` was changed manually, or the `tox.jinja`
+                template and/or the `populate_tox.py` script were changed without
+                regenerating `tox.ini`.
+
+                Please don't make manual changes to `tox.ini`. Instead, make the
+                changes to the `tox.jinja` template and/or the `populate_tox.py`
+                script (as applicable) and regenerate the `tox.ini` file with:
+
+                python -m venv toxgen.env
+                . toxgen.env/bin/activate
+                pip install -r scripts/populate_tox/requirements.txt
+                python scripts/populate_tox/populate_tox.py
+                """
+                )
+            )
+        print("Done checking tox.ini. Looking good!")
+    else:
+        print(
+            "Done generating tox.ini. Make sure to also update the CI YAML files to reflect the new test targets."
+        )
 
 
 if __name__ == "__main__":
-    main()
+    fail_on_changes = len(sys.argv) == 2 and sys.argv[1] == "--fail-on-changes"
+    main(fail_on_changes)
